@@ -1,5 +1,5 @@
 ;;;; Copyright (C) 1996 by Gilbert Baumann
-;;;; Copyright (C) 2001-2007 by Sam Steingold
+;;;; Copyright (C) 2001-2008 by Sam Steingold
 ;;;; Distributed under GPL.
 ;;;; Some parts are from the MIT-CLX Distribution, copyrighted by
 ;;;; Texas Instruments Incorporated, but freely distributable
@@ -170,7 +170,12 @@
    display-trace ; for backwards compatibility describe-request describe-event describe-reply
    closed-display-p
    ;; extensions
-   open-default-display
+   open-default-display with-open-display
+   display-get-default display-resource-manager-string screen-resource-string
+   *canonicalize-encoding*
+   ;;; Only when using libXt:
+   ;; last-event-processed last-timestamp-processed
+   ;;; Only when using the native resource database, not resource.lisp:
    ;; display-xdefaults resource-database-to-string resource-database-of-string
    ;; resource-database-locale
    ;; not implemented
@@ -339,7 +344,7 @@
                     (:constructor nil)
                     (:copier nil)
                     (:conc-name %display-))
-  foreign-pointer ;; these two slots are for use in clx.d only.
+  foreign-pointer ;; these two slots are for use in clx.f only.
   hash-table      ;; .. so keep hands off here!
   plist
   after-function
@@ -540,25 +545,25 @@
         dashes? clip-mask?)
     (do ((q options (cddr q)))
         ((null q))
-        (cond ((eq (car q) :dashes)    (setf dashes? t))
-              ((eq (car q) :clip-mask) (setf clip-mask? t)))
-        (setf comps      (logior comps (%gcontext-key->mask (car q)))
-              setf-forms (nconc setf-forms
-                                (list (list (find-symbol (string-concat "GCONTEXT-" (symbol-name (car q))) :xlib)
-                                            gcon)
-                                      (cadr q)))) )
+      (cond ((eq (car q) :dashes)    (setf dashes? t))
+            ((eq (car q) :clip-mask) (setf clip-mask? t)))
+      (setf comps      (logior comps (%gcontext-key->mask (car q)))
+            setf-forms (nconc setf-forms
+                              (list (list (find-symbol (ext:string-concat "GCONTEXT-" (symbol-name (car q))) :xlib)
+                                          gcon)
+                                    (cadr q)))))
     `(LET* ((,gcon ,gcontext)
             (,saved (%SAVE-GCONTEXT-COMPONENTS ,gcon ,comps))
             ,@(if dashes?    (list `(,g0 (GCONTEXT-DASHES    ,gcon))))
-            ,@(if clip-mask? (list `(,g1 (GCONTEXT-CLIP-MASK ,gcon)))) )
+            ,@(if clip-mask? (list `(,g1 (GCONTEXT-CLIP-MASK ,gcon)))))
        (UNWIND-PROTECT
-           (PROGN
-             (SETF ,@setf-forms)
-             ,@body)
+            (PROGN
+              (SETF ,@setf-forms)
+              ,@body)
          (PROGN
            (%RESTORE-GCONTEXT-COMPONENTS ,gcon ,saved)
-           ,@(if dashes?    (list `(SETF (GCONTEXT-DASHES ,gcon) ,g0)) )
-           ,@(if clip-mask? (list `(SETF (GCONTEXT-CLIP-MASK ,gcon) ,g1)) )))) ))
+           ,@(if dashes?    (list `(SETF (GCONTEXT-DASHES ,gcon) ,g0)))
+           ,@(if clip-mask? (list `(SETF (GCONTEXT-CLIP-MASK ,gcon) ,g1))))))))
 
 (defmacro WITH-SERVER-GRABBED ((display) &body body)
   ;; The body is not surrounded by a with-display.
@@ -600,23 +605,29 @@
   `(set-string-property ,window :WM_CLIENT_MACHINE ,name))
 
 (defun get-wm-class (window)
-  (let ((value (get-property window :WM_CLASS :type :STRING :result-type 'string :transform #'card8->char)))
+  (let ((value (get-property window :WM_CLASS :type :STRING
+                             :result-type 'string :transform #'card8->char)))
     (when value
-      (let* ((name-len (position (load-time-value (card8->char 0)) (the string value)))
-             (name (subseq (the string value) 0 name-len))
-             (class (subseq (the string value) (1+ name-len) (1- (length value)))))
+      ;; Buggy clients may not comply with the format,
+      ;; so deal with the unexpected.
+      (let* ((first-zero (position #\Null (the string value)))
+             (second-zero (and first-zero
+                               (position #\Null (the string value)
+                                         :start (1+ first-zero))))
+             (name (subseq (the string value) 0 first-zero))
+             (class (and first-zero
+                         (subseq (the string value) (1+ first-zero)
+                                 second-zero))))
         (values (and (plusp (length name)) name)
                 (and (plusp (length class)) class))))))
 
 (defun set-wm-class (window resource-name resource-class)
   (set-string-property window :WM_CLASS
-                       (string-concat
-                        (string (or resource-name ""))
-                        (load-time-value
-                         (make-string 1 :initial-element (card8->char 0)))
-                        (string (or resource-class ""))
-                        (load-time-value
-                         (make-string 1 :initial-element (card8->char 0)))))
+                       (ext:string-concat
+                        (string (or resource-name #1=""))
+                        #2=#,(string #\Null)
+                        (string (or resource-class #1#))
+                        #2#))
   (values))
 
 (defun wm-command (window)
@@ -629,7 +640,7 @@
         (end 0)
         (len (length command-string)))
        ((>= start len) (nreverse command))
-    (setq end (position (load-time-value (card8->char 0)) command-string :start start))
+    (setq end (position #\Null command-string :start start))
     (push (subseq command-string start end) command)))
 
 (defsetf wm-command set-wm-command)
@@ -645,7 +656,7 @@
       (with-standard-io-syntax
         (dolist (c command)
           (prin1 c stream)
-          (write-char (load-time-value (card8->char 0)) stream)))))
+          (write-char #\Null stream)))))
   command)
 
 ;;-----------------------------------------------------------------------------
@@ -653,7 +664,8 @@
 
 ;; Some of the functions below need decode-type and encode-type,
 ;; I provide here a limited implementation to get these functions working.
-;;
+;; sds: I am pretty sure that these should be eliminated and the code using
+;;      them should be moved to C for the same reasong (set-)wm-hints had to
 (defmacro decode-type (type value)
   (cond ((eq type 'pixmap) `(lookup-pixmap %buffer ,value))
         ((eq type 'window) `(lookup-window %buffer ,value))
@@ -673,6 +685,7 @@
         (t (error "Unknown type ~S." type)) ))
 
 (defstruct wm-hints
+  ;; keep in sync with clx.f
   (input nil )
   (initial-state nil )
   (icon-pixmap nil )
@@ -685,89 +698,7 @@
   ;; may be extended in the future
   )
 
-(defun wm-hints (window)
-  (let ((prop (get-property window :WM_HINTS :type :WM_HINTS
-                            :result-type 'vector)))
-    (when prop
-      (decode-wm-hints prop (window-display window)))))
-
 (defsetf wm-hints set-wm-hints)
-(defun set-wm-hints (window wm-hints)
-  (change-property window :WM_HINTS (encode-wm-hints wm-hints) :WM_HINTS 32)
-  wm-hints)
-
-(defun decode-wm-hints (vector display)
-  (let ((input-hint 0)
-        (state-hint 1)
-        (icon-pixmap-hint 2)
-        (icon-window-hint 3)
-        (icon-position-hint 4)
-        (icon-mask-hint 5)
-        (window-group-hint 6))
-    (let ((flags (aref vector 0))
-          (hints (make-wm-hints))
-          (%buffer display))
-      (setf (wm-hints-flags hints) flags)
-      (when (logbitp input-hint flags)
-        (setf (wm-hints-input hints) (decode-type (member :off :on)
-                                                  (aref vector 1))))
-      (when (logbitp state-hint flags)
-        (setf (wm-hints-initial-state hints)
-              (decode-type (member :dont-care :normal :zoom :iconic :inactive)
-                           (aref vector 2))))
-      (when (logbitp icon-pixmap-hint flags)
-        (setf (wm-hints-icon-pixmap hints) (decode-type pixmap (aref vector 3))))
-      (when (logbitp icon-window-hint flags)
-        (setf (wm-hints-icon-window hints) (decode-type window (aref vector 4))))
-      (when (logbitp icon-position-hint flags)
-        (setf (wm-hints-icon-x hints) (aref vector 5)
-              (wm-hints-icon-y hints) (aref vector 6)))
-      (when (logbitp icon-mask-hint flags)
-        (setf (wm-hints-icon-mask hints) (decode-type pixmap (aref vector 7))))
-      (when (and (logbitp window-group-hint flags) (> (length vector) 7))
-        (setf (wm-hints-window-group hints) (aref vector 8)))
-      hints)))
-
-(defun encode-wm-hints (wm-hints)
-  (let ((input-hint         #b1)
-        (state-hint         #b10)
-        (icon-pixmap-hint   #b100)
-        (icon-window-hint   #b1000)
-        (icon-position-hint #b10000)
-        (icon-mask-hint     #b100000)
-        (window-group-hint  #b1000000)
-        (mask               #b1111111)
-        )
-    (let ((vector (make-array 9 :initial-element 0))
-          (flags 0))
-      (declare (type (simple-vector 9) vector)
-               (type card16 flags))
-      (when (wm-hints-input wm-hints)
-        (setf flags input-hint
-              (aref vector 1) (encode-type (member :off :on) (wm-hints-input wm-hints))))
-      (when (wm-hints-initial-state wm-hints)
-        (setf flags (logior flags state-hint)
-              (aref vector 2) (encode-type (member :dont-care :normal :zoom :iconic :inactive)
-                                           (wm-hints-initial-state wm-hints))))
-      (when (wm-hints-icon-pixmap wm-hints)
-        (setf flags (logior flags icon-pixmap-hint)
-              (aref vector 3) (encode-type pixmap (wm-hints-icon-pixmap wm-hints))))
-      (when (wm-hints-icon-window wm-hints)
-        (setf flags (logior flags icon-window-hint)
-              (aref vector 4) (encode-type window (wm-hints-icon-window wm-hints))))
-      (when (and (wm-hints-icon-x wm-hints) (wm-hints-icon-y wm-hints))
-        (setf flags (logior flags icon-position-hint)
-              (aref vector 5) (encode-type card16 (wm-hints-icon-x wm-hints))
-              (aref vector 6) (encode-type card16 (wm-hints-icon-y wm-hints))))
-      (when (wm-hints-icon-mask wm-hints)
-        (setf flags (logior flags icon-mask-hint)
-              (aref vector 7) (encode-type pixmap (wm-hints-icon-mask wm-hints))))
-      (when (wm-hints-window-group wm-hints)
-        (setf flags (logior flags window-group-hint)
-              (aref vector 8) (wm-hints-window-group wm-hints)))
-      (setf (aref vector 0) (logior flags (logandc2 (wm-hints-flags wm-hints) mask)))
-      vector)))
-
 
 ;;-----------------------------------------------------------------------------
 ;; WM_SIZE_HINTS
@@ -1390,6 +1321,15 @@ default display as given by GET-DEFAULT-DISPLAY otherwise."
        (setf (display-default-screen dpy) screen)
        dpy)))
 
+(defmacro with-open-display ((display &rest options) &body body)
+  "Open a DISPLAY, execute BODY, close the DISPLAY."
+  `(let ((,display ,(if options
+                        `(open-display ,@options)
+                        `(open-default-display))))
+     (unwind-protect (progn ,@body)
+       (when ,display
+         (close-display ,display)))))
+
 
 ;;;; --------------------------------------------------------------------------
 ;;;;  Stuff, which is realy some internals of CLX,
@@ -1835,3 +1775,15 @@ default display as given by GET-DEFAULT-DISPLAY otherwise."
 (undefined TRANSLATE-DEFAULT)
 (undefined QUEUE-EVENT)
 )
+
+;; canonicalize encodings supplied by X, see clx.f:get_font_info_and_display()
+(defvar *canonicalize-encoding*
+  ;; this encoding canonicalization was requested by
+  ;; Pascal J.Bourguignon <pjb@informatimago.com>
+  ;; in <http://article.gmane.org/gmane.lisp.clisp.general:7794>
+  `(,(lambda (s)
+       (if (and (<= 4 (length s))
+                (string-equal s "iso" :end1 3)
+                (not (char= #\- (char s 3))))
+           (concatenate 'string "ISO-" (subseq s 3))
+           s))))
