@@ -1,6 +1,6 @@
 /*
  * CLISP: directory key: win32 registry, LDAP, Gnome-config
- * Copyright (C) 2000-2003 by Sam Steingold
+ * Copyright (C) 2000-2007 by Sam Steingold
  */
 
 /* have to undefing UNICODE _here_ because clisp.h will #include <windows.h> */
@@ -107,9 +107,7 @@ enum { /* DIR-KEY slots */
 
 /* check whether the OBJ is a DIR-KEY */
 static gcv_object_t* dir_key_slots (object obj) {
-  if (!structurep(obj) ||
-      nullp(memq(`LDAP::DIR-KEY`,TheStructure(obj)->structure_types)))
-    return NULL;
+  if (!typep_classname(obj,`LDAP::DIR-KEY`)) return NULL;
   return TheStructure(obj)->recdata+1; /* FIXME for derived structs! */
 }
 
@@ -124,13 +122,13 @@ static object test_dir_key (object obj, bool check_open) {
       pushSTACK(`LDAP::DIR-KEY`); /* TYPE-ERROR slot EXPECTED-TYPE */
       pushSTACK(`LDAP::DIR-KEY`); pushSTACK(obj);
       pushSTACK(TheSubr(subr_self)->name);
-      check_value(type_error,GETTEXT("~: ~ is not a ~"));
+      check_value(type_error,GETTEXT("~S: ~S is not a ~S"));
       obj = value1; goto start;
     }
     if (check_open && nullp(slots[DK_OPEN])) {
       pushSTACK(obj);
       pushSTACK(TheSubr(subr_self)->name);
-      check_value(error,GETTEXT("~ on ~ is illegal"));
+      check_value(error_condition,GETTEXT("~S on ~S is illegal"));
       obj = value1; goto start;
     }
     XOUT(obj,"directory key");
@@ -354,15 +352,15 @@ static void open_reg_key (HKEY hkey, char* path, direction_t dir,
 #endif
 
 #if defined(ACCESS_LDAP)
-nonreturning_function(static, fehler_ldap,
+nonreturning_function(static, error_ldap,
                       (object dk, object path, char* errmsg)) {
   end_system_call();
   pushSTACK(NIL); pushSTACK(path); pushSTACK(dk);
   pushSTACK(TheSubr(subr_self)->name);
   STACK_3 = CLSTEXT(errmsg);
-  fehler(error,"~(~ ~): ~");
+  error(error_condition,"~S(~S ~S): ~S");
 }
-#define LDAP_ERR2STR(d,p,status) fehler_ldap(d,p,ldap_err2string(status))
+#define LDAP_ERR2STR(d,p,status) error_ldap(d,p,ldap_err2string(status))
 #define LDAP_RES2STR(d,p,ld,res) LDAP_ERR2STR(d,p,ldap_result2error(ld,res,1))
 #endif
 
@@ -427,7 +425,7 @@ DEFUN(LDAP::DIR-KEY-OPEN, key path &key DIRECTION IF-DOES-NOT-EXIST) {
         end_system_call();
         pushSTACK(path);
         pushSTACK(TheSubr(subr_self)->name);
-        fehler(error,GETTEXT("~: ~ is not an LDAP URL"));
+        error(error_condition,GETTEXT("~S: ~S is not an LDAP URL"));
       }
       ret_handle = (void*)ldap_open(ldap_url->lud_host,ldap_url->lud_port);
       if (ret_handle == NULL) OS_error();
@@ -454,7 +452,7 @@ DEFUN(LDAP::DIR-KEY-OPEN, key path &key DIRECTION IF-DOES-NOT-EXIST) {
     pushSTACK(`LDAP::DIR-KEY`);
     pushSTACK(type);
     pushSTACK(TheSubr(subr_self)->name);
-    fehler(type_error,GETTEXT("~: ~ is not a ~"));
+    error(type_error,GETTEXT("~S: ~S is not a ~S"));
   }
   /* create the DIR-KEY */
   pushSTACK(type);
@@ -594,7 +592,7 @@ static scope_t parse_scope (object scope) {
   pushSTACK(`:SCOPE`);
   pushSTACK(scope);
   pushSTACK(TheSubr(subr_self)->name);
-  fehler(type_error,GETTEXT("~: ~ is not a ~"));
+  error(type_error,GETTEXT("~S: ~S is not a ~S"));
 }
 
 /* return a simple vector with the iteration state
@@ -646,14 +644,15 @@ static void init_iteration_node (object state, object subkey,
   test_dir_key(ITST_DKEY(STACK_4),true);
  {object dk = test_dir_key(ITST_DKEY(STACK_4/*state*/),true);
   gcv_object_t *slots = dir_key_slots(dk);
-  FOREIGN *fp = &(TheFpointer(STACK_0/*handle*/)->fp_pointer);
+  FOREIGN fp;
   with_string_0(*new_path,GLO(misc_encoding),pathz,{
     open_reg_key((HKEY)SLOT_HANDLE(slots),pathz,check_direction(slots[DK_DIR]),
-                 IF_DOES_NOT_EXIST_UNBOUND/*ignore*/,(HKEY*)fp);
+                 IF_DOES_NOT_EXIST_UNBOUND/*ignore*/,(HKEY*)&fp);
   });
-  if (*fp) {
+  TheFpointer(STACK_0/*handle*/)->fp_pointer = fp;
+  if (fp) {
     DWORD k_size, a_size, d_size;
-    SYSCALL_WIN32(RegQueryInfoKey((HKEY)*fp,NULL,NULL,NULL,NULL,&k_size,
+    SYSCALL_WIN32(RegQueryInfoKey((HKEY)fp,NULL,NULL,NULL,NULL,&k_size,
                                   NULL,NULL,&a_size,&d_size,NULL,NULL));
     NODE_KEY_S(STACK_1) = fixnum(k_size+1); /* node */
     NODE_ATT_S(STACK_1) = fixnum(a_size+1); /* node */
@@ -676,8 +675,8 @@ static object state_next_key (object state) {
   object stack = ITST_STACK(state);
   if (!nullp(stack)) {
     object node = ITST_NODE(state);
-    uintL keynum = posfixnum_to_L(NODE_KEY(node));
-    uintL keylen = posfixnum_to_L(NODE_KEY_S(node));
+    uintL keynum = posfixnum_to_V(NODE_KEY(node));
+    uintL keylen = posfixnum_to_V(NODE_KEY_S(node));
     char* buffer = (char*)alloca(keylen);
     Fpointer fp  = TheFpointer(NODE_HANDLE(node));
     XOUT(state,"state_next_key");
@@ -751,13 +750,13 @@ DEFUN(LDAP::DKEY-SEARCH-NEXT-ATT,state)
     pushSTACK(`LDAP::DKEY-SEARCH-NEXT-KEY`);
     pushSTACK(state);
     pushSTACK(`LDAP::DKEY-SEARCH-NEXT-ATT`);
-    fehler(error,GETTEXT("~ from ~ without ~ before it"));
+    error(error_condition,GETTEXT("~S from ~S without ~S before it"));
   }
  {object node = Car(stack);
   Fpointer fp = TheFpointer(NODE_HANDLE(node));
-  uintL attnum = posfixnum_to_L(NODE_ATT(node));
-  uintL attlen = posfixnum_to_L(NODE_ATT_S(node));
-  uintL datlen = posfixnum_to_L(NODE_DAT_S(node));
+  uintL attnum = posfixnum_to_V(NODE_ATT(node));
+  uintL attlen = posfixnum_to_V(NODE_ATT_S(node));
+  uintL datlen = posfixnum_to_V(NODE_DAT_S(node));
   char* att = (char*)alloca(attlen);
   char* dat = (char*)alloca(datlen);
   DWORD type;
@@ -857,7 +856,7 @@ DEFUN(LDAP::SET-DKEY-VALUE, key name value)
     } else {
       pushSTACK(value);
       pushSTACK(TheSubr(subr_self)->name);
-      fehler(error,GETTEXT("~ on ~ is illegal"));
+      error(error_condition,GETTEXT("~S on ~S is illegal"));
     }
   });
   VALUES1(value);
@@ -922,14 +921,7 @@ DEFUN(LDAP::DKEY-INFO,key) {
   value6 = L_to_I(max_value_name_length);
   value7 = L_to_I(max_value_length);
   value8 = L_to_I(security_descriptor);
-# if defined(WIN32_NATIVE)
-  value9 = convert_time_to_universal(&write_time);
-# else
-  {
-    time_t unix_time = to_time_t_(&write_time);
-    value9 = convert_time_to_universal(&unix_time);
-  }
-# endif
+  value9 = convert_time_to_universal_w32(&write_time);
   mv_count = 9;
 }
 
