@@ -93,13 +93,6 @@ typedef SOCKET rawsock_t;
 DEFMODULE(rawsock,"RAWSOCK")
 
 /* ================== helpers ================== */
-/* can trigger GC */
-static object my_check_argument (object name, object datum) {
-  pushSTACK(NIL);               /* no PLACE */
-  pushSTACK(name); pushSTACK(datum); pushSTACK(TheSubr(subr_self)->name);
-  check_value(error_condition,GETTEXT("~S: ~S is not a valid ~S argument"));
-  return value1;
-}
 /* DANGER: the return value is invalidated by GC!
  > *arg_: vector
  > STACK_0, STACK_1: END & START -- removed
@@ -209,7 +202,7 @@ DEFUN(RAWSOCK:SOCKADDR-FAMILY, sa) {
   VALUES2(fixnum(sa->sa_family),fixnum(size)); skipSTACK(1);
 }
 DEFUN(RAWSOCK::SOCKADDR-SLOT,&optional slot) {
-  /* return offset & size of the slo in SOCKADDR */
+  /* return offset & size of the slot in SOCKADDR */
  restart_sockaddr_slot:
   if (missingp(STACK_0)) {
     VALUES1(fixnum(sizeof(struct sockaddr)));
@@ -250,7 +243,7 @@ static object sockaddr_to_lisp (struct sockaddr *sa, uintL size) {
   end_system_call();
   funcall(`RAWSOCK::MAKE-SA`,1); return value1;
 }
-#define sockaddr_to_lisp1(sa)  sockaddr_to_lisp(sa,sizeof(*sa))
+#define pushSTACK_sockaddr(sa)  pushSTACK(sa ? sockaddr_to_lisp(sa,sizeof(*sa)) : NIL)
 
 struct pos {
   gcv_object_t *vector;
@@ -265,8 +258,7 @@ void coerce_into_bytes (void *arg, object element) {
 
 DEFUN(RAWSOCK:MAKE-SOCKADDR,family &optional data) {
   int family = check_socket_domain(STACK_1);
-  struct sockaddr sa;
-  unsigned char *buffer, *data;
+  unsigned char *buffer;
   size_t buffer_len, data_start = offsetof(struct sockaddr,sa_data);
   struct pos arg;
   if (missingp(STACK_0)) {      /* standard size */
@@ -561,33 +553,39 @@ DEFUN(RAWSOCK:IF-NAME-INDEX, &optional what) {
 #endif  /* net/if.h */
 /* ================== ifaddrs.h interface ================== */
 #if defined(HAVE_NET_IF_H) && defined(HAVE_IFADDRS_H) && defined(HAVE_GETIFADDRS) && defined(HAVE_FREEIFADDRS)
-DEFCHECKER(check_iff,prefix=IFF,bitmasks=both,:UP BROADCAST DEBUG LOOPBACK \
+DEFCHECKER(check_iff,prefix=IFF,bitmasks=both,default=(unsigned)~0,     \
+           :UP BROADCAST DEBUG LOOPBACK                                 \
            POINTOPOINT NOTRAILERS RUNNING NOARP PROMISC ALLMULTI        \
-           OACTIVE SIMPLEX LINK0 LINK1 LINK2 ALTPHYS POLLING PPROMISC MONITOR \
-           STATICARP NEEDSGIANT                                         \
-           MASTER SLAVE MULTICAST PORTSEL AUTOMEDIA DYNAMIC)
-DEFUN(RAWSOCK:IFADDRS,) {
+           OACTIVE SIMPLEX LINK0 LINK1 LINK2 ALTPHYS POLLING PPROMISC   \
+           MONITOR STATICARP NEEDSGIANT                                 \
+           MASTER SLAVE MULTICAST PORTSEL AUTOMEDIA DYNAMIC LOWER-UP DORMANT)
+DEFUN(RAWSOCK:IFADDRS,&key flags-and flags-or) {
   struct ifaddrs *ifap;
-  int count;
+  int count = 0;
+  unsigned int flags_or = check_iff_of_list(popSTACK());
+  unsigned int flags_and = missingp(STACK_0) ? 0 : check_iff_of_list(STACK_0);
+  skipSTACK(1);                 /* drop flags_and */
   begin_system_call(); if(-1==getifaddrs(&ifap)) OS_error(); end_system_call();
-  for (count=0; ifap; ifap=ifap->ifa_next, count++) {
-    pushSTACK(asciz_to_string(ifap->ifa_name,GLO(misc_encoding)));
-    pushSTACK(check_iff_to_list(ifap->ifa_flags));
-    pushSTACK(ifap->ifa_addr ? sockaddr_to_lisp1(ifap->ifa_addr) : NIL);
-    pushSTACK(ifap->ifa_netmask ? sockaddr_to_lisp1(ifap->ifa_netmask) : NIL);
-    if (ifap->ifa_flags & IFF_BROADCAST)
-      if (ifap->ifa_flags & IFF_POINTOPOINT) {
-        pushSTACK(TheSubr(subr_self)->name);
-        error(error_condition,GETTEXT("~S: both IFF_BROADCAST and IFF_POINTOPOINT set"));
-      } else pushSTACK(ifap->ifa_broadaddr
-                       ? sockaddr_to_lisp1(ifap->ifa_broadaddr) : NIL);
-    else if (ifap->ifa_flags & IFF_POINTOPOINT)
-      pushSTACK(ifap->ifa_dstaddr ? sockaddr_to_lisp1(ifap->ifa_dstaddr) : NIL);
-    else pushSTACK(NIL);
-    pushSTACK(ifap->ifa_data ? allocate_fpointer(ifap->ifa_data) : NIL);
-    funcall(`RAWSOCK::MAKE-IFADDRS`,6);
-    pushSTACK(value1);
-  }
+  for (; ifap; ifap=ifap->ifa_next)
+    if ((flags_or & ifap->ifa_flags)
+        && ((flags_and & ifap->ifa_flags) == flags_and)) {
+      pushSTACK(asciz_to_string(ifap->ifa_name,GLO(misc_encoding)));
+      pushSTACK(check_iff_to_list(ifap->ifa_flags));
+      pushSTACK_sockaddr(ifap->ifa_addr);
+      pushSTACK_sockaddr(ifap->ifa_netmask);
+      if (ifap->ifa_flags & IFF_BROADCAST)
+        if (ifap->ifa_flags & IFF_POINTOPOINT) {
+          pushSTACK(STACK_3);   /* ifa_name */
+          pushSTACK(TheSubr(subr_self)->name);
+          error(error_condition,GETTEXT("~S: both IFF_BROADCAST and IFF_POINTOPOINT set for ~S"));
+        } else pushSTACK_sockaddr(ifap->ifa_broadaddr);
+      else if (ifap->ifa_flags & IFF_POINTOPOINT)
+        pushSTACK_sockaddr(ifap->ifa_dstaddr);
+      else pushSTACK(NIL);
+      pushSTACK(ifap->ifa_data ? allocate_fpointer(ifap->ifa_data) : NIL);
+      funcall(`RAWSOCK::MAKE-IFADDRS`,6);
+      pushSTACK(value1); count++;
+    }
   begin_system_call(); freeifaddrs(ifap); end_system_call();
   VALUES1(listof(count));
 }
@@ -1496,6 +1494,7 @@ DEFUN(RAWSOCK:UDPCSUM, buffer &key :START :END) { /* UDP checksum */
 
 void module__rawsock__init_function_2 (module_t* module);
 void module__rawsock__init_function_2 (module_t* module) {
+  unused module;
 #if defined(WIN32_NATIVE)
   HMODULE ws2 = LoadLibrary("ws2_32.dll");
   if (ws2 != NULL) {
