@@ -278,9 +278,9 @@ Use the usual editing capabilities.
    (TEXT "
 Commands may be abbreviated as shown in the second column.
 COMMAND        ABBR     DESCRIPTION
-Help           :h, ?    this command list
-Error          :e       Print the last error message
-Inspect        :i       Inspect the last error
+Help           :h, ?    print this command list
+Error          :e       print the last error message
+Inspect        :i       inspect the last error
 Abort          :a       abort to the next recent input loop
 Unwind         :uw      abort to the next recent input loop
 Reset          :re      toggle *PACKAGE* and *READTABLE* between the
@@ -410,7 +410,7 @@ Continue       :c       switch off single step mode, continue evaluation
        ,@body)))
 
 ;; Main-Loop with additional help-command
-(defun main-loop ()
+(defun main-loop (&optional (exit t))
   (setq *break-count* 0)
   (driver                 ; build driver-frame; do #'lambda "infinitely"
    #'(lambda ()
@@ -422,7 +422,13 @@ Continue       :c       switch off single step mode, continue evaluation
            ;; T -> #<EOF>
            ;; NIL -> form is already evaluated
            ;;        result has been printed
-           (exit))))))
+           (if exit
+               ;; if you want to change this logic, you need to make sure that
+               ;; 1. "clisp -x (1+ 2) -repl" prints "3" and then the prompt
+               ;; 2. "cat script | clisp" and "clisp < script" do not hang
+               (exit)
+               (progn (setq *command-index* 0) ; reset *command-index*
+                      (return-from main-loop))))))))
 
 (setq *driver* #'main-loop)
 
