@@ -187,7 +187,7 @@ resolve_shell_symlink (LPCSTR filename, LPSTR resolved)
 
 /* the ultimate shortcut megaresolver
    style inspired by directory_search_scandir
- > namein: filename pointing to file or directory
+ > namein: absolute filename pointing to file or directory
             wildcards (only asterisk) may appear only as filename
  < nameout: filename with directory and file shortcuts resolved
              on failure holds filename resolved so far
@@ -203,6 +203,7 @@ BOOL real_path (LPCSTR namein, LPSTR nameout) {
   char saved_char;
   BOOL next_name = 0;/* if we found an lnk and need to start over */
   int try_counter = 33;
+  if (strlen(namein) >= MAX_PATH) return FALSE;
   strcpy(nameout,namein);
   do { /* whole file names */
     next_name = FALSE;
@@ -237,7 +238,7 @@ BOOL real_path (LPCSTR namein, LPSTR nameout) {
       /* save slash or zero */
       saved_char = *nametocheck_end;
       *nametocheck_end = 0;
-      /* Is it . or .. ? FF handles this strange way */
+      /* Is it . or .. ? FFF handles this strange way */
       { char * cp = nametocheck;
         for (;*cp=='.';cp++);
         dots_only = !(*cp) && cp > nametocheck; }
@@ -249,15 +250,16 @@ BOOL real_path (LPCSTR namein, LPSTR nameout) {
       if (have_stars && saved_char) return FALSE;
       if (!have_stars) {
         if (dots_only || !*nametocheck) {
-          /* treat 'start/./end' and 'drive|host|slash/' specially */
+          /* treat 'start/./end', 'drive/', 'host/' specially */
           /* search for ....\.\* */
           char saved[2];
           if (nametocheck_end - nameout + 2 > MAX_PATH) return FALSE;
-          strncpy(saved,nametocheck_end+1,2);
-          strncpy(nametocheck_end,"\\*",3);
+          saved[0] = nametocheck_end[1]; saved[1] = nametocheck_end[2];
+          /* !*nametocheck here means there was "something\" before */
+          strcpy(nametocheck_end,*nametocheck?"\\*":"*");
           h = FindFirstFile(nameout,&wfd);
-          strncpy(nametocheck_end+1,saved,2);
-          *nametocheck_end = 0;
+          nametocheck_end[1] = saved[0]; nametocheck_end[2] = saved[1];
+          nametocheck_end[0] = 0;
           if (h != INVALID_HANDLE_VALUE) {
             FindClose(h); /* don't substitute */
           } else return FALSE; /* don't try lnk */
