@@ -1,11 +1,46 @@
 ;; run the test suit:
 
+;;; --- helpers for misc tests ---
 (defun princ-error (c) (format t "~&[~A]: ~A~%" (type-of c) c))
-
+(defun show (object &key ((:pretty *print-pretty*) *print-pretty*))
+  "Print the object on its own line and return it. Used in many tests!"
+  (fresh-line) (prin1 object) (terpri) object)
+#+clisp (progn
+(defun kill-down (name)
+  (dolist (f (directory (ext:string-concat name "**/*")))
+    (format t "~&removing file ~S~%" f)
+    (delete-file f)))
+(defun rmrf (name)
+  (ext:dir (ext:string-concat name "**/*"))
+  (ext:dir (ext:string-concat name "**/"))
+  (kill-down name)
+  (dolist (d (directory (ext:string-concat name "**/")))
+    (format t "~&removing directory ~S~%" d)
+    (ext:delete-directory d)))
+(defun prepare-directory (name)
+  (ensure-directories-exist name :verbose t)
+  (kill-down name))
+(defun cmd-args ()
+  "command and arg list for cloning this clisp process"
+  (let* ((argv (ext:argv))
+         (lispinit (aref argv (1+ (position "-M" argv :test #'string=)))))
+    (values (aref argv 0)
+            (list "-q" "-norc" "-B" (namestring *lib-directory*) "-M"
+                  (namestring (merge-pathnames lispinit *lib-directory*))))))
+)
+(defun show-file (file)         ; return line count
+  (with-open-file (st file :direction :input)
+    (format t "~&~S: ~:D byte~:P:~%" file (file-length st))
+    (loop :for l = (read-line st nil nil) :while l :count t
+      :do (format t "--> ~S~%" l))))
+(defun finish-file (file) (prog1 (show-file file) (delete-file file)))
 (defun post-compile-file-cleanup (file)
   (delete-file file)
   (delete-file (compile-file-pathname file))
   #+clisp (delete-file (make-pathname :type "lib" :defaults file)))
+(defun symbol-cleanup (s)
+  (setf (find-class s) nil) (makunbound s) (fmakunbound s) (unintern s))
+;;; end helpers
 
 #+OLD-CLISP
 ;; Binding *ERROR-HANDLER* is a hammer technique for catching errors. It also
@@ -124,10 +159,6 @@
                 slot s-r lisp-implementation s-m (equal s-r s-m)))))
   (:method ((result t) (my-result t) (log stream)))) ; do nothing
 
-(defun show (object &key ((:pretty *print-pretty*) *print-pretty*))
-  "Print the object on its own line and return it. Used in many tests!"
-  (fresh-line) (prin1 object) (terpri) object)
-
 (defun type-error-handler (err)
   "Print the condition and THROW.
 Usage: (handler-bind ((type-error #'type-error-handler)) ...)"
@@ -261,16 +292,16 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
   (if (zerop error-count)
       (delete-file logfile)
       (format t "~s: see ~a~%" 'run-test logfile))
-  (list testname total-count error-count))
+  (list logname total-count error-count))
 
 (defun report-results (res)
   "res = list of RUN-TEST return values (testname total-count error-count)"
   (let ((error-count (reduce #'+ res :key #'third)))
     (format
-     t "~&finished ~3d file~:p:~31T ~3:d error~:p out of~50T ~5:d test~:p~%"
+     t "~&finished ~3d file~:p:~31T ~3:d error~:p out of~50T ~6:d test~:p~%"
      (length res) error-count (reduce #'+ res :key #'second))
     (loop :with here = (truename "./") :for rec :in res :for count :upfrom 1 :do
-      (format t "~&~3d ~25@a:~31T ~3:d error~:p out of~50T ~5:d test~:p~%"
+      (format t "~&~3d ~25@a:~31T ~3:d error~:p out of~50T ~6:d test~:p~%"
               count (enough-namestring (first rec) here)
               (third rec) (second rec)))
     error-count))
@@ -344,8 +375,7 @@ NIL: sacla-style: forms should evaluate to non-NIL.")
                   #+(or CLISP ALLEGRO CMU19 OpenMCL LISPWORKS) "weakhash"
                   #+(or CLISP LISPWORKS) "weakhash2"))
       (push (run-test ff) res))
-    (unless disable-risky ; fails on amd64 - disable for now...
-      (push (run-test "bind" :eval-method :eval :logname "bind-eval") res))
+    (push (run-test "bind" :eval-method :eval :logname "bind-eval") res)
     (push (run-test "bind" :eval-method :compile :logname "bind-compile") res)
     #+(or CLISP ALLEGRO CMU SBCL LISPWORKS)
     (let ((tmp (list "weakptr" 0 0)))
